

#version 3.7;

#include "math.inc"
#include "./materials.inc"
#include "./dvdlogo.inc"
#include "./path.inc"
#include "./changes.inc"

#declare TOTAL_FRAMES = 2880.0;

#declare X = 50;
#declare Y = 50;
#declare Z = 50;

#declare POS_MULT = -1.5;
#declare FOCAL_MULT = -0.25;

#declare CAM_Z = -200;
#declare FRAME = floor(clock * TOTAL_FRAMES);

#declare colorChoice = Changes[FRAME];
#declare color1 = <LogoColors[colorChoice][0], LogoColors[colorChoice][1], LogoColors[colorChoice][2]>;


#declare plastic = texture{
  pigment { rgb color1 }
  finish {
    ambient 0
    diffuse .06
  }
}

global_settings {
  assumed_gamma 2.2
  photons {
    count 200000
    autostop 0
    jitter .4
  }
}

#declare useDOF=false;

#if (useDOF)
	#debug "Using DOF rendering"
	camera {
		//orthographic
		sky <0,0,1>
		location <0, 0, CAM_Z>
		look_at <0, 0, 0>
		angle 50
		right -x*(image_width/image_height)
		aperture 5.6
		blur_samples 200
		variance 1/10000
		focal_point <0, 0, 0>
	}
#else
	#debug "Not using DOF rendering"
	camera {
		//orthographic
		sky <0,0,1>
		location <0, 0, CAM_Z>
		look_at <0, 0, 0>
		angle 50
		right -x*(image_width/image_height)
		focal_point <0, 0, 0>
	}
#end

light_source {
  <0, -60-Y, -100> 
  rgb <0.1, 0.1, 0.1>
  area_light x,y,6,6
  area_illumination off
}

#macro r(c) pigment{rgb c} finish {phong 0.3 ambient 0.5 diffuse 0.5} #end
plane { y, -1 
  texture{ r(<0, 0, 0>)} 
  rotate y*50 
  translate <5*X, 5*Y, Z>
  finish { ambient 1 diffuse 0}
}

object {
  pov_dvdlogo
  Center_Trans(pov_dvdlogo, x+y+z)
  translate <Path[FRAME][1], Path[FRAME][0], 0>
  rotate <0, 0, -90>
  texture{ 
    plastic
  }
}